@@ -23,7 +23,7 @@ using namespace std::string_literals;
 
 namespace fs = std::filesystem;
 
-float to_px(const cssparser::Dimension &value)
+inline float to_px(const cssparser::Dimension &value)
 {
     switch (value.unit) {
     case cssparser::Unit::Px:
@@ -33,7 +33,7 @@ float to_px(const cssparser::Dimension &value)
     }
 }
 
-float to_px(const cssparser::Value &value)
+inline float to_px(const cssparser::Value &value)
 {
     if (!std::holds_alternative<cssparser::Dimension>(value)) {
         return 0.0;
@@ -89,7 +89,7 @@ Color to_color(const cssparser::Color::Color &color)
     return Color{};
 }
 
-Color to_color(const cssparser::Value &value)
+inline Color to_color(const cssparser::Value &value)
 {
     if (!std::holds_alternative<cssparser::Color::Color>(value)) {
         return Color{};
@@ -98,7 +98,7 @@ Color to_color(const cssparser::Value &value)
     return to_color(std::get<cssparser::Color::Color>(value));
 }
 
-fs::path to_path(const cssparser::Value &value)
+inline fs::path to_path(const cssparser::Value &value)
 {
     if (!std::holds_alternative<cssparser::Url>(value)) {
         return fs::path{};
@@ -108,12 +108,22 @@ fs::path to_path(const cssparser::Value &value)
     return url.data;
 }
 
+bool matches_keyword(const cssparser::Value &value, const QString &keyword)
+{
+    if (!std::holds_alternative<std::string>(value)) {
+        return false;
+    }
+
+    auto string = QString::fromStdString(std::get<std::string>(value));
+    return string.compare(keyword, Qt::CaseInsensitive) == 0;
+}
+
 template<class... Ts>
 struct overloads : Ts... {
     using Ts::operator()...;
 };
 
-QVariant to_qvariant(const cssparser::Value &value)
+inline QVariant to_qvariant(const cssparser::Value &value)
 {
     constexpr auto visitor = overloads{
         /* clang-format off */
@@ -151,6 +161,34 @@ inline T toEnumValue(const std::string &value)
     return T{toEnumIntValue<T>(value)};
 }
 
+template<typename T>
+inline void setImage(T &output, const fs::path &rootPath, const cssparser::Property &property)
+{
+    QImage imageData;
+    std::filesystem::path path = rootPath / to_path(property.value());
+
+    if (!imageData.load(QString::fromStdString(path))) {
+        qCWarning(UNION_CSS) << "Could not load image" << path.string();
+        return;
+    }
+
+    auto image = output.image_or_new();
+    image.setImageData(imageData);
+    image.setWidth(imageData.width());
+    image.setHeight(imageData.height());
+    if (property.values.size() > 1) {
+        auto value = property.values.at(1);
+        if (matches_keyword(value, u"mask"_s)) {
+            image.setFlags(ImageFlag::Mask);
+        } else if (matches_keyword(value, u"inverted-mask"_s)) {
+            image.setFlags(ImageFlag::InvertedMask);
+        }
+        image.setMaskColor(to_color(property.values.at(2)));
+    }
+
+    output.setImage(image);
+}
+
 template<typename T>
 inline void setAlignment(T &output, const cssparser::Property &property)
 {
@@ -231,12 +269,21 @@ void setDirectionValue(T &output, const std::string &baseName, const cssparser::
 
     if (property.name == baseName) {
         directions = {"left", "right", "top", "bottom"};
-        properties = {"width", "color"};
-        values = {property.value(0), property.value(2)};
+        properties = {"width", "style", "color"};
+
+        if (property.values.size() == 1 && std::holds_alternative<std::string>(property.value())) {
+            values = {std::nullopt, property.value(), std::nullopt};
+        } else {
+            values = {property.value(0), property.value(1), property.value(2)};
+        }
     } else if (property.name == baseName + "-width") {
         directions = {"left", "right", "top", "bottom"};
         properties = {"width"};
         values = {property.value()};
+    } else if (property.name == baseName + "-style") {
+        directions = {"left", "right", "top", "bottom"};
+        properties = {"style"};
+        values = {property.value()};
     } else if (property.name == baseName + "-color") {
         directions = {"left", "right", "top", "bottom"};
         properties = {"color"};
@@ -256,8 +303,8 @@ void setDirectionValue(T &output, const std::string &baseName, const cssparser::
             properties = {matches.captured(3).toLatin1()};
             values = {property.value()};
         } else {
-            properties = {"width", "color"};
-            values = {property.value(0), property.value(2)};
+            properties = {"width", "style", "color"};
+            values = {property.value(0), property.value(1), property.value(2)};
         }
     }
 
@@ -266,6 +313,8 @@ void setDirectionValue(T &output, const std::string &baseName, const cssparser::
             line.setSize(to_px(value));
         } else if (property == "size") {
             line.setSize(to_px(value));
+        } else if (property == "style") {
+            line.setStyle(toEnumValue<Union::Properties::LineStyle>(std::get<std::string>(value)));
         } else if (property == "color") {
             line.setColor(to_color(value));
         }
@@ -467,23 +516,28 @@ void CssLoader::setBackgroundProperty(StyleProperty &output, const cssparser::Pr
 {
     auto background = output.background_or_new();
 
+    if (property.name == "background"s) {
+        auto value = property.value();
+        if (matches_keyword(value, u"none"_s)) {
+            background.setColor(Union::Color{});
+            background.setImage(ImageProperty::empty());
+        } else if (std::holds_alternative<cssparser::Color::Color>(value)) {
+            background.setColor(to_color(value));
+        } else if (std::holds_alternative<cssparser::Url>(value)) {
+            setImage(background, m_stylePath, property);
+        }
+    }
+
     if (property.name == "background-color"s) {
         background.setColor(to_color(property.value()));
     }
 
     if (property.name == "background-image") {
-        auto path = m_stylePath / to_path(property.value());
-
-        QImage imageData;
-        if (!imageData.load(QString::fromStdString(path))) {
-            qCWarning(UNION_CSS) << "Could not load image" << path.string();
+        if (matches_keyword(property.value(), u"none"_s)) {
+            background.setImage(ImageProperty::empty());
+        } else {
+            setImage(background, m_stylePath, property);
         }
-
-        auto image = background.image_or_new();
-        image.setImageData(imageData);
-        image.setWidth(imageData.width());
-        image.setHeight(imageData.height());
-        background.setImage(image);
     }
 
     if (background.hasAnyValue()) {
@@ -634,14 +688,18 @@ void CssLoader::setShadowProperty(StyleProperty &output, const cssparser::Proper
     auto shadow = output.shadow_or_new();
 
     if (property.name == "box-shadow") {
-        auto offset = shadow.offset_or_new();
-        offset.setHorizontal(to_px(property.value(0)));
-        offset.setVertical(to_px(property.value(1)));
-        shadow.setOffset(offset);
+        if (matches_keyword(property.value(), u"none"_s)) {
+            shadow = ShadowProperty::empty();
+        } else {
+            auto offset = shadow.offset_or_new();
+            offset.setHorizontal(to_px(property.value(0)));
+            offset.setVertical(to_px(property.value(1)));
+            shadow.setOffset(offset);
 
-        shadow.setBlur(to_px(property.value(2)));
-        shadow.setSize(to_px(property.value(3)));
-        shadow.setColor(to_color(property.value(4)));
+            shadow.setBlur(to_px(property.value(2)));
+            shadow.setSize(to_px(property.value(3)));
+            shadow.setColor(to_color(property.value(4)));
+        }
     }
 
     if (shadow.hasAnyValue()) {
